package lsjsample;

import java.io.Serializable;
import java.lang.Integer;
import java.lang.Long;
import java.lang.Short;
import java.lang.String;
import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.Date;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;

import jp.littlesoft.data.ITableDef;
import jp.littlesoft.data.ITableDef.TableDef;
import jp.littlesoft.data.exception.DataException;
import jp.littlesoft.sql.SchemaUpdator;
import jp.littlesoft.util.DataUtil;
import jp.littlesoft.web.WebSQLTable;

/**
 * Little J-Programing Sample
 * サンプルデータベース エンティティクラス
 * 
 * @author LittleSoft Corporation
 * @version 1.3.0
 */
@SuppressWarnings("serial")
public class DB implements Serializable {

  public DB() {}
  
  @TableDef(name="M_DEPT", schema="LSJSAMPLE")
  static public class Department implements Serializable, ITableDef {
    @ColumnDef(name="DEPT_CODE", caption="所属コード")
    public Integer code;
    @ColumnDef(name="DEPT_NAME", caption="所属名称")
    public String name;
    @Override
    public MetaData<?> createMetaData() {
      return new MetaData<Department>() {{
        addPrimaryKey($$.code);
      }};
    }
  }
  
  @TableDef(name="M_WORK_TYPE", schema="LSJSAMPLE")
  static public class WorkType implements Serializable, ITableDef {
    @ColumnDef(name="WORK_TYPE_CODE", caption="出勤区分コード")
    public Short code;
    @ColumnDef(name="WORK_TYPE_NAME", caption="出勤区分名称")
    public String name;
    public MetaData<?> createMetaData() {
      return new MetaData<WorkType>() {{
        addPrimaryKey($$.code);
      }};
    }
  }
  
  @TableDef(name="T_EMP", schema="LSJSAMPLE")
  static public class Employee implements Serializable, ITableDef {
    @ColumnDef(name="EMP_ID", caption="社員ID")
    public Long id;
    @ColumnDef(name="DEPT_CODE", caption="所属")
    public Integer deptCode;
    @ColumnDef(name="NAME", caption="氏名")
    public String name;
    @ColumnDef(name="ENTER_DATE", caption="入社日", formatPattern="yyyy/MM/dd")
    public Date enterDate;
    @ColumnDef(name="SEX", caption="性別", defaultValue="1")
    public Short sex;
    @ColumnDef(name="SALARY", caption="給与", formatPattern="#,##0.00;-#,##0.00")
    public BigDecimal salary;
    @ColumnDef(name="MEMO", caption="メモ", defaultValue="中途／新卒")
    public String memo;
    @ColumnDef(name="CREATE_DATETIME", caption="登録日時")
    public Timestamp createDatetime;
    @ColumnDef(name="UPDATE_DATETIME", caption="更新日時", formatPattern=DataUtil.XWareki.FORMAT_DATETIME_SHORT)
    public Timestamp updateDatetime;
    
    public MetaData<?> createMetaData() {
      return new MetaData<Employee>() {{
        addPrimaryKey($$.id);
      }};
    }
  }
  
  @TableDef(name="T_EMP_SEQ", schema="LSJSAMPLE")
  static public class EmployeeSeq implements Serializable, ITableDef {
    @ColumnDef(name="SEQ")
    public Long value;
    public MetaData<?> createMetaData() {
      return new MetaData<EmployeeSeq>();
    }
  }
  
  @TableDef(name="T_INFO", schema="LSJSAMPLE")
  static public class Information implements Serializable, ITableDef {
    @ColumnDef(name="INFO_ID")
    public Integer id;
    @ColumnDef(name="INFO_TEXT", caption="お知らせ内容")
    public String text;
    
    public MetaData<?> createMetaData() {
      return new MetaData<Information>() {{
        addPrimaryKey($$.id);
      }};
    }
  }
  
  @TableDef(name="T_WORK", schema="LSJSAMPLE")
  static public class Work implements Serializable, ITableDef {
    @ColumnDef(name="WORK_ID", caption="勤務情報ID")
    public Long id;
    @ColumnDef(name="EMP_ID", caption="社員ID")
    public Long empId;
    @ColumnDef(name="WORK_DATE", caption="日付" , formatPattern="yyyy/MM/dd")
    public Date date;
    @ColumnDef(name="WORK_TYPE_CODE", caption="出勤区分")
    public Short workTypeCode;
    @ColumnDef(name="BEGIN_TIME_M", caption="開始時刻")
    public Integer beginTimeM;
    @ColumnDef(name="FINISH_TIME_M", caption="終了時刻")
    public Integer finishTimeM;
    @ColumnDef(name="BREAK_TIME_M", caption="休憩時間")
    public Integer breakTimeM;
    @ColumnDef(name="MEMO", caption="メモ")
    public String memo;
    @ColumnDef(name="CREATE_DATETIME", caption="登録日時", formatPattern=DataUtil.XWareki.FORMAT_DATETIME_SHORT )
    public Timestamp createDatetime;
    @ColumnDef(name="UPDATE_DATETIME", caption="更新日時", formatPattern=DataUtil.XWareki.FORMAT_DATETIME_SHORT )
    public Timestamp updateDatetime;
    
    public MetaData<?> createMetaData() {
      return new MetaData<Work>() {{
        addPrimaryKey($$.id);
      }};
    }
  }
  
  @TableDef(name="T_WORK_SEQ", schema="LSJSAMPLE")
  static public class WorkSeq implements Serializable, ITableDef {
    @ColumnDef(name="SEQ")
    public Long value;
    
    public MetaData<?> createMetaData() {
      return new MetaData<WorkSeq>();
    }
  }

  public void createOrAlter(Connection conn) throws DataException {
    new SchemaUpdator(conn).createOrAlterTable(
        DB.Department.class,
        DB.WorkType.class,
        DB.Employee.class,
        DB.EmployeeSeq.class,
        DB.Information.class,
        DB.Work.class,
        DB.WorkSeq.class
    );
  }
  public void insertInitializeData(Connection conn) throws DataException {
    try {
      java.sql.Timestamp createTimestamp = new java.sql.Timestamp(System.currentTimeMillis());
      // 初期データ登録（T_INFO）
      WebSQLTable<Information> info = new WebSQLTable<Information>() {{
        select().prepare();
      }};
      info.emptyActivate(conn);
      info.appendRow();
      info.entity.id   = 1;
      info.entity.text = "<span style=\"font-size: large\" class=\"Apple-style-span\"><span style=\"font-style: italic\" class=\"Apple-style-span\"><span style=\"color: #008080\" class=\"Apple-style-span\">LittleSoft J-Programming</span></span></span><div><span style=\"font-size: 18px\" class=\"Apple-style-span\"><br /></span><div>軽量AJAX+Javaフレームワーク LittleSoft J-Programming(LSJ)がGoogle App Engineに対応しました。</div><div>当サイトは、LSJによるサンプルアプリケーションをご紹介しています。</div></div>";
      info.resolveChanges(conn);
      // 初期データ登録（M_DEPT）
      WebSQLTable<Department> dept = new WebSQLTable<Department>() {{
        select().prepare();
      }};
      dept.emptyActivate(conn);
      dept.appendRow();
      dept.entity.code = 1;
      dept.entity.name = "営業部";
      dept.appendRow();
      dept.entity.code = 2;
      dept.entity.name = "人事部";
      dept.appendRow();
      dept.entity.code = 3;
      dept.entity.name = "開発部";
      dept.resolveChanges(conn);
      // 初期データ登録（T_EMP）
      WebSQLTable<Employee> emp = new WebSQLTable<Employee>() {{
        select().prepare();
      }};
      emp.emptyActivate(conn);
      emp.appendRow();
      emp.entity.id = new Long(1);
      emp.entity.deptCode = 1;
      emp.entity.name = "磯野波平";
      emp.entity.enterDate = new java.sql.Date(new SimpleDateFormat("yyyy-MM-dd").parse("1997-09-14").getTime());
      emp.entity.sex = 1;
      emp.entity.salary = new BigDecimal(250000);
      emp.entity.memo = "会社員";
      emp.entity.createDatetime = createTimestamp;
      emp.entity.updateDatetime = createTimestamp;
      emp.appendRow();
      emp.entity.id = new Long(2);
      emp.entity.deptCode = 1;
      emp.entity.name = "タマ";
      emp.entity.enterDate = new java.sql.Date(new SimpleDateFormat("yyyy-MM-dd").parse("2007-12-01").getTime());
      emp.entity.sex = 1;
      emp.entity.salary = new BigDecimal(250000);
      emp.entity.memo = "ペット";
      emp.entity.createDatetime = createTimestamp;
      emp.entity.updateDatetime = createTimestamp;
      emp.appendRow();
      emp.entity.id = new Long(3);
      emp.entity.deptCode = 2;
      emp.entity.name = "波野タイ子";
      emp.entity.enterDate = new java.sql.Date(new SimpleDateFormat("yyyy-MM-dd").parse("2003-02-08").getTime());
      emp.entity.sex = 2;
      emp.entity.salary = new BigDecimal(250000);
      emp.entity.memo = "専業主婦";
      emp.entity.createDatetime = createTimestamp;
      emp.entity.updateDatetime = createTimestamp;
      emp.appendRow();
      emp.entity.id = new Long(4);
      emp.entity.deptCode = 1;
      emp.entity.name = "磯野ワカメ";
      emp.entity.enterDate = new java.sql.Date(new SimpleDateFormat("yyyy-MM-dd").parse("2004-03-11").getTime());
      emp.entity.sex = 2;
      emp.entity.salary = new BigDecimal(250000);
      emp.entity.memo = "小学生";
      emp.entity.createDatetime = createTimestamp;
      emp.entity.updateDatetime = createTimestamp;
      emp.appendRow();
      emp.entity.id = new Long(5);
      emp.entity.deptCode = 1;
      emp.entity.name = "フグ田サザエ";
      emp.entity.enterDate = new java.sql.Date(new SimpleDateFormat("yyyy-MM-dd").parse("2001-12-16").getTime());
      emp.entity.sex = 2;
      emp.entity.salary = new BigDecimal(250000);
      emp.entity.memo = "専業主婦";
      emp.entity.createDatetime = createTimestamp;
      emp.entity.updateDatetime = createTimestamp;
      emp.appendRow();
      emp.entity.id = new Long(6);
      emp.entity.deptCode = 1;
      emp.entity.name = "磯野フネ";
      emp.entity.enterDate = new java.sql.Date(new SimpleDateFormat("yyyy-MM-dd").parse("1999-10-11").getTime());
      emp.entity.sex = 2;
      emp.entity.salary = new BigDecimal(250000);
      emp.entity.memo = "専業主婦";
      emp.entity.createDatetime = createTimestamp;
      emp.entity.updateDatetime = createTimestamp;
      emp.appendRow();
      emp.entity.id = new Long(7);
      emp.entity.deptCode = 1;
      emp.entity.name = "磯野カツオ";
      emp.entity.enterDate = new java.sql.Date(new SimpleDateFormat("yyyy-MM-dd").parse("2002-10-11").getTime());
      emp.entity.sex = 1;
      emp.entity.salary = new BigDecimal(250000);
      emp.entity.memo = "小学生";
      emp.entity.createDatetime = createTimestamp;
      emp.entity.updateDatetime = createTimestamp;
      emp.appendRow();
      emp.entity.id = new Long(8);
      emp.entity.deptCode = 1;
      emp.entity.name = "フグ田マスオ";
      emp.entity.enterDate = new java.sql.Date(new SimpleDateFormat("yyyy-MM-dd").parse("2000-06-23").getTime());
      emp.entity.sex = 1;
      emp.entity.salary = new BigDecimal(250000);
      emp.entity.memo = "会社員";
      emp.entity.createDatetime = createTimestamp;
      emp.entity.updateDatetime = createTimestamp;
      emp.appendRow();
      emp.entity.id = new Long(9);
      emp.entity.deptCode = 1;
      emp.entity.name = "フグ田タラオ";
      emp.entity.enterDate = new java.sql.Date(new SimpleDateFormat("yyyy-MM-dd").parse("2006-04-09").getTime());
      emp.entity.sex = 1;
      emp.entity.salary = new BigDecimal(250000);
      emp.entity.memo = "小学生未満";
      emp.entity.createDatetime = createTimestamp;
      emp.entity.updateDatetime = createTimestamp;
      emp.appendRow();
      emp.entity.id = new Long(10);
      emp.entity.deptCode = 2;
      emp.entity.name = "波野ノリスケ";
      emp.entity.enterDate = new java.sql.Date(new SimpleDateFormat("yyyy-MM-dd").parse("2001-08-24").getTime());
      emp.entity.sex = 1;
      emp.entity.salary = new BigDecimal(250000);
      emp.entity.memo = "会社員";
      emp.entity.createDatetime = createTimestamp;
      emp.entity.updateDatetime = createTimestamp;
      emp.appendRow();
      emp.entity.id = new Long(11);
      emp.entity.deptCode = 2;
      emp.entity.name = "波野イクラ";
      emp.entity.enterDate = new java.sql.Date(new SimpleDateFormat("yyyy-MM-dd").parse("2007-01-01").getTime());
      emp.entity.sex = 1;
      emp.entity.salary = new BigDecimal(250000);
      emp.entity.memo = "子供";
      emp.entity.createDatetime = createTimestamp;
      emp.entity.updateDatetime = createTimestamp;
      emp.appendRow();
      emp.entity.id = new Long(12);
      emp.entity.deptCode = 3;
      emp.entity.name = "伊佐坂難物";
      emp.entity.enterDate = new java.sql.Date(new SimpleDateFormat("yyyy-MM-dd").parse("1993-08-29").getTime());
      emp.entity.sex = 1;
      emp.entity.salary = new BigDecimal(250000);
      emp.entity.memo = "恋愛小説家";
      emp.entity.createDatetime = createTimestamp;
      emp.entity.updateDatetime = createTimestamp;
      emp.appendRow();
      emp.entity.id = new Long(13);
      emp.entity.deptCode = 3;
      emp.entity.name = "伊佐坂軽";
      emp.entity.enterDate = new java.sql.Date(new SimpleDateFormat("yyyy-MM-dd").parse("1995-10-30").getTime());
      emp.entity.sex = 2;
      emp.entity.salary = new BigDecimal(250000);
      emp.entity.memo = "専業主婦";
      emp.entity.createDatetime = createTimestamp;
      emp.entity.updateDatetime = createTimestamp;
      emp.appendRow();
      emp.entity.id = new Long(14);
      emp.entity.deptCode = 3;
      emp.entity.name = "伊佐坂浮江";
      emp.entity.enterDate = new java.sql.Date(new SimpleDateFormat("yyyy-MM-dd").parse("2003-05-01").getTime());
      emp.entity.sex = 2;
      emp.entity.salary = new BigDecimal(250000);
      emp.entity.memo = "高校生";
      emp.entity.createDatetime = createTimestamp;
      emp.entity.updateDatetime = createTimestamp;
      emp.appendRow();
      emp.entity.id = new Long(15);
      emp.entity.deptCode = 3;
      emp.entity.name = "伊佐坂甚六";
      emp.entity.enterDate = new java.sql.Date(new SimpleDateFormat("yyyy-MM-dd").parse("2001-11-12").getTime());
      emp.entity.sex = 1;
      emp.entity.salary = new BigDecimal(250000);
      emp.entity.memo = "浪人生";
      emp.entity.createDatetime = createTimestamp;
      emp.entity.updateDatetime = createTimestamp;
      emp.appendRow();
      emp.entity.id = new Long(16);
      emp.entity.deptCode = 3;
      emp.entity.name = "ハチ公";
      emp.entity.enterDate = new java.sql.Date(new SimpleDateFormat("yyyy-MM-dd").parse("2007-04-10").getTime());
      emp.entity.sex = 1;
      emp.entity.salary = new BigDecimal(250000);
      emp.entity.memo = "ペット（犬）";
      emp.entity.createDatetime = createTimestamp;
      emp.entity.updateDatetime = createTimestamp;
      emp.resolveChanges(conn);
      // 初期データ登録（M_WORK_TYPE）
      WebSQLTable<WorkType> wType = new WebSQLTable<WorkType>() {{
        select().prepare();
      }};
      wType.emptyActivate(conn);
      wType.appendRow();
      wType.entity.code = 1;
      wType.entity.name = "出勤";
      wType.appendRow();
      wType.entity.code = 2;
      wType.entity.name = "欠勤";
      wType.resolveChanges(conn);
      // 初期データ登録（T_EMP_SEQ）
      WebSQLTable<EmployeeSeq> eSeq = new WebSQLTable<EmployeeSeq>() {{
        select().prepare();
      }};
      eSeq.emptyActivate(conn);
      eSeq.appendRow();
      eSeq.entity.value = new Long(16);
      eSeq.resolveChanges(conn);
      // 初期データ登録（T_WORK_SEQ）
      WebSQLTable<WorkSeq> wSeq = new WebSQLTable<WorkSeq>() {{
        select().prepare();
      }};
      wSeq.emptyActivate(conn);
      wSeq.appendRow();
      wSeq.entity.value = new Long(0);
      wSeq.resolveChanges(conn);
      // コミット
      conn.commit();
    }
    catch (Exception x) {
      try { conn.rollback(); } catch (Exception e) {}
      x.printStackTrace();
    }
  }
  
  public boolean checkExistsDb(Connection conn) throws SQLException {
    DatabaseMetaData md = conn.getMetaData();
    ResultSet rs = md.getSchemas();
    boolean isSchemaExist = false;
    while (rs.next()) {
      if (rs.getString(1).equals("LSJSAMPLE")) {
        isSchemaExist = true;
        break;
      }
    }
    rs.close();
    return isSchemaExist;
  }
}
