package lsjsample;

import java.io.IOException;
import java.sql.Connection;
//import java.sql.DriverManager;

import javax.naming.InitialContext;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import java.util.HashMap;
import java.util.logging.Logger;

import jp.littlesoft.sql.exception.LSSQLException;
// For [ GAE ]
//import jp.littlesoft.sql4g.GAdministrator;
//import jp.littlesoft.sql4g.GEngine;

/**
 * Little J-Programing Sample
 * メインサーブレット
 *
 * @author LittleSoft Corporation
 * @version 1.3.0
 */
@SuppressWarnings("serial")
public class MainServlet extends jp.littlesoft.web.ActionController {
  private Logger logger = Logger.getLogger(this.getClass().getName());
  protected String characterEncoding = "UTF-8";
  public String realPath;
  // For [ Tomcat ]
  static private HashMap<String, DataSource> activeDataSources = new HashMap<String, DataSource>();
  // For [ GAE ]
//  static private HashMap<String, GAdministrator> activeServices = new HashMap<String, GAdministrator>();
  
  /**
   * サーブレットイニシャライズメソッド
   *
   * @see jp.littlesoft.web.ActionController#init(javax.servlet.ServletConfig)
   */
  @Override
  public void init(ServletConfig config) throws ServletException {
    super.init(config);
    // For [ GAE ]
//    try {
//      Class.forName("jp.littlesoft.sql4g.Driver");
//    } catch (ClassNotFoundException e) {
//      e.printStackTrace();
//      throw new ServletException(e);
//    }
  }
  
  /**
   * サービスメソッド
   * Encodingなどデルフォルト設定を行います。
   *
   * @see jp.littlesoft.web.ActionController#service(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
   */
  @Override
  protected void service(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
    request.setCharacterEncoding(characterEncoding);
    response.setCharacterEncoding(characterEncoding);
    request.setAttribute("realPath", realPath);
    super.service(request, response);
  }
  
  /**
   * セッション開始処理
   *
   * @see jp.littlesoft.web.ActionController#onStartNewSession(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
   */
  @Override
  protected void onStartNewSession(HttpServletRequest request, HttpServletResponse response) throws Exception {
    logger.info("### onStartNewSession ###");
  }
  
  /**
   * エラー処理
   *
   * @see jp.littlesoft.web.ActionController#errorAction(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse, java.lang.Exception)
   */
  @Override
  protected void errorAction(HttpServletRequest request, HttpServletResponse response, Exception x) throws ServletException, IOException{
    x.printStackTrace();
    try {
      java.io.PrintWriter pw = response.getWriter();
      x.printStackTrace(pw);
      pw.flush();
      pw.close();
    } catch (Exception e) {
      throw new ServletException(e);
    }
  }
  
  /**
   * javax.sql.DataSourceを検索し、JDBCコネクションを取得します。
   *
   * @see jp.littlesoft.web.ActionController#getConnection()
   */
  @Override
  public Connection getConnection(HttpServletRequest request) throws LSSQLException {
    try {
      // For [ Tomcat ]
      String dsName = "java:/comp/env/jdbc/lsjsample";
      if (!activeDataSources.containsKey(dsName)) {
        InitialContext  dsInitContext = new InitialContext();
        DataSource ds = (DataSource) dsInitContext.lookup(dsName);
        Connection conn = ds.getConnection();
        DB db = new DB();
        boolean isSchemaExist = db.checkExistsDb(conn);
        db.createOrAlter(conn);
        if (!isSchemaExist) {
          db.insertInitializeData(conn);
        }
        activeDataSources.put(dsName, ds);
        return conn;
      }
      else {
        DataSource ds = activeDataSources.get(dsName);
        return ds.getConnection();
      }
      // For [ GAE ]
//      String key = "LS_SAMPLEDB";
//      if (!activeServices.containsKey(key)) {
//        GAdministrator admin = checkApplicationDB();
//        activeServices.put(key, admin);
//        return admin.getConnection();
//      }
//      else {
//        return DriverManager.getConnection(key, "SA", "ls");
//      }
    }
    catch (Exception e) {
      throw new LSSQLException(e);
    }
  }

  @Override
  public void destroy() {
    logger.info("### destroy() ###");
    super.destroy();
  }

  @Override
  protected void finalize() throws Throwable {
    logger.info("### finalize() ###");
    super.finalize();
  }

  // For [ GAE ]
//  static public GAdministrator checkApplicationDB() throws Exception  {
//    GAdministrator admin;
//    DB db = new DB();
//    java.io.PrintWriter pw = new java.io.PrintWriter(System.out);
//    try {
//      admin = new GAdministrator("LS_SAMPLEDB", "SA", "ls");
//      while(!admin.startup(pw));
//      db.createOrAlter(admin.getConnection());
//      admin.checkPoint(pw);
//    }
//    catch (GEngine.DatabaseNotFoundException x) {
//      admin = new GAdministrator("LS_SAMPLEDB", "SA", "ls", true);
//      admin.checkPoint(pw);
//      db.createOrAlter(admin.getConnection());
//      db.insertInitializeData(admin.getConnection());
//      admin.checkPoint(pw);
//    }
//    return admin;
//  }
}
