package lsjsample;

import java.sql.Connection;
import java.text.SimpleDateFormat;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import jp.littlesoft.LSApplicationException;
import jp.littlesoft.web.SQLActionPage;
import jp.littlesoft.web.WebSQLTable;

/**
 * Little J-Programing Sample
 * ようこそ画面
 * 
 * @author LittleSoft Corporation
 * @version 1.3.0
 */
@SuppressWarnings("serial")
public class WelcomePage extends SQLActionPage {

  /**
   * ログイン情報(現在時刻)
   */
  public String now = null;

  /**
   * お知らせが編集状態か否か
   */
  public boolean isEditInfomation = false;
  
  /**
   * WebSQLTableオブジェクト(お知らせ情報テーブル)
   * SELECT * FROM T_INFO WHERE T_INFO.INFO_ID = :1
   */
  public WebSQLTable<DB.Information> tableMemo = new WebSQLTable<DB.Information>(){{
    select().where($$.id).is(new Integer(1)).end.prepare();
  }};

  /**
   * Velocityテンプレートファイル(ようこそ画面)
   */
  private static String welcomeTemplate = "WelcomePage.html";
  
  /**
   * コンストラクタ。
   * 
   * @throws Exception 例外発生時
   */
  public WelcomePage() {
  }
  
  /**
   * コンストラクタ。
   * 
   * @param request リクエスト
   * @param response レスポンス
   * @throws Exception 例外発生時
   */
  public WelcomePage(HttpServletRequest request, HttpServletResponse response) throws Exception {
    
    super(request, response);
    setTemplateName(welcomeTemplate); //HTMLテンプレートを指定します。
    setStrictSession(true);           //バックボタンによる不正なリクエストをキャンセルし、最終ページを強制出力する指定です。
    //データベース読み込み
    loadDatabase();
  }

  /**
   * データベースを読み込みます。
   * 
   * @throws Exception 例外発生時
   */
  private void loadDatabase() throws Exception {
    //コネクション取得
    Connection conn = getConnection();
    try {
      //データベースを検索し、結果をWebSQLTableオブジェクトに格納する
      tableMemo.activate(conn);  
      
      if (tableMemo.getRowCount()==0) {
        DB.Information info = new DB.Information();
        info.id = 1;
        info.text = "あたらしい記事";
        tableMemo.validate(info);
      }
    } finally {
      //コネクションを閉じる
      conn.close(); 
    }
  }
  
  /**
   * 各アクションメソッド開始前に呼ばれるメソッドです。
   * 
   * @param methodName メソッド名
   */
  @Override
  protected void onStartAction(String methodName) {
    SimpleDateFormat sdf = new SimpleDateFormat("yyyy'年'MM'月'dd'日'(E) HH:mm:ss");
    now = sdf.format(new java.util.Date());
  }
  
  /**
   * 「お知らせ」の編集を行います。
   * 
   * @throws Exception 例外発生時
   */
  public void editInfomation() throws Exception {
    isEditInfomation = true;
    show();
  }

  /**
   * 「お知らせ」の登録を行います。
   * 
   * @throws Exception 例外発生時
   */
  public void saveInfomation() throws Exception {
    // コネクション取得
    Connection conn = getConnection();
    try {
      // データベース更新
      tableMemo.resolveChanges(conn);
      conn.commit();
      isEditInfomation = false;
    } catch (Exception x) {
      x.printStackTrace();
      conn.rollback();
      // 例外発生時はWebSQLTableにエラーメッセージを追加
      tableMemo.addErrorMessage(x.getLocalizedMessage());
      throw new LSApplicationException(x);
    } finally {
      // コネクションを閉じて再表示
      conn.close();
      show();
    }
  }
  
}
