package lsjsample;

import java.io.Serializable;
import java.sql.Connection;
import java.sql.Date;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import jp.littlesoft.LSApplicationException;
import jp.littlesoft.data.Row;
import jp.littlesoft.data.event.TableEvent;
import jp.littlesoft.sql.SQL;
import jp.littlesoft.web.SQLActionPage;
import jp.littlesoft.web.WebSQLTable;

/**
 * Little J-Programing Sample
 * 勤務表の参照(ダイアログ)画面
 *
 * @author LittleSoft Corporation
 * @version 1.3.0
 */
@SuppressWarnings("serial")
public class WorkDialog extends SQLActionPage {

  /**
   * 時間リスト(セレクトボックス)
   */
  public List<Map<String, String>> hourList = getHourList();

  /**
   * 分リスト(セレクトボックス)
   */
  public List<Map<String, String>> minuteList = getMinuteList();

  /**
   * 編集対象の社員ID
   */
  public Long empId = null;

  /**
   * 編集対象の日付
   */
  public String date = "";

  /**
   * 勤務時間
   */
  public String workTime = "00：00";

  /**
   * 作業時間編集ビュー
   * @author littlesoft
   *
   */
  static public class WorkEdit extends WorkPage.WorkView {

    private int calcMinute(int hour, int minute) {
      return hour * 60 + minute;
    }

    @ColumnDef( formatPattern="00" )
    public Integer getBeginTimeHour() {
      return beginTimeM / 60;
    }

    public void setBeginTimeHour(Integer beginTimeHour) {
      beginTimeM = calcMinute(beginTimeHour, getBeginTimeMinute());
    }

    @ColumnDef( formatPattern="00" )
    public Integer getBeginTimeMinute() {
      return beginTimeM % 60;
    }

    public void setBeginTimeMinute(Integer beginTimeMinute) {
      beginTimeM = calcMinute(getBeginTimeHour(), beginTimeMinute);
    }

    @ColumnDef( formatPattern="00" )
    public Integer getFinishTimeHour() {
      return finishTimeM / 60;
    }

    public void setFinishTimeHour(Integer finishTimeHour) {
      finishTimeM = calcMinute(finishTimeHour, getFinishTimeMinute());
    }

    @ColumnDef( formatPattern="00" )
    public Integer getFinishTimeMinute() {
      return finishTimeM % 60;
    }

    void setFinishTimeMinute(Integer finishTimeMinute) {
      finishTimeM = calcMinute(getFinishTimeHour(), finishTimeMinute);
    }

    @ColumnDef( formatPattern="00" )
    public Integer getBreakTimeHour() {
      return breakTimeM / 60;
    }

    public void setBreakTimeHour(Integer breakTimeHour) {
      breakTimeM = calcMinute(breakTimeHour, getBreakTimeMinute());
    }

    @ColumnDef( formatPattern="00" )
    public Integer getBreakTimeMinute() {
      return breakTimeM % 60;
    }

    public void setBreakTimeMinute(Integer breakTimeMinute) {
      breakTimeM = calcMinute(getBreakTimeHour(), breakTimeMinute);
    }
  }
  /**
   * WebSQLTableオブジェクト(勤務情報テーブル)
   */
  public WebSQLTable<WorkEdit> tableWorkEdit = new WebSQLTable<WorkEdit>() {{
    DB.Work work = $useDummy(DB.Work.class);
    select($(work).into($$))
    .from(work).where(work.empId).is(":EMP_ID")
      .and(work.date).is(":WORK_DATE")
    .end
    .prepare();
  }};

  /**
   * WebSQLTableオブジェクト(社員情報テーブル)
   * SELECT EMP_ID, NAME, DEPT_NAME
   *   FROM T_EMP E INNER JOIN M_DEPT D
   *     ON E.DEPT_CODE = D.DEPT_CODE WHERE E.EMP_ID = :EMP_ID");
   */
  static public class EmployeeView  implements Serializable {
    public Long id;
    public String name;
    public String deptName;
  }

  public WebSQLTable<EmployeeView> tableWorkEmp = new WebSQLTable<EmployeeView>(){{
    DB.Employee   e = $useDummy(DB.Employee.class);
    DB.Department d = $useDummy(DB.Department.class);
    select(
      $(e.id).into($$.id),
      $(e.name).into($$.name),
      $(d.name).into($$.deptName)
    )
    .from(e).innerJoin(d).on(e.deptCode).is(d.code).end
    .where(e.id).is(":EMP_ID").end
    .prepare();
  }};

  /**
   * WebSQLTableオブジェクト(勤務情報採番テーブル)
   * SELECT SEQ FROM T_WORK_SEQ
   */
  public WebSQLTable<DB.WorkSeq> tableWorkSeq = new WebSQLTable<DB.WorkSeq>(){{
    select().prepare();
  }};

  /**
   * UpdateSQLオブジェクト(勤務情報採番テーブル)
   * UPDATE T_WORK_SEQ SET SEQ = SEQ + 1
   */
  public SQL<DB.WorkSeq> updateWorkSeq = new SQL<DB.WorkSeq>() {{
    update().set(
      $($$.value).accept( $($$.value).add(1) )
    ).prepare();
  }};

  /**
   * Velocityテンプレートファイル(勤務情報編集ダイアログ)
   */
  private static String workEditTemplate = "WorkDialog.html";

  /**
   * コンストラクタ。
   *
   * @throws Exception 例外発生時
   */
  public WorkDialog() throws Exception {

    setUseAjax(true);                   // Ajax呼び出しを使用します。
    setTemplateName(workEditTemplate);  // HTMLテンプレートを指定します。
    setStrictSession(true);             // バックボタンによる不正なリクエストをキャンセルします。
    /*
     * 勤務情報テーブルオブジェクト(tableWorkEdit)の設定
     */
    tableWorkEdit.addTableListener(this);
    $column(tableWorkEdit.$$.id).setNullable(true);
    $column(tableWorkEdit.$$.beginTimeM).setNullable(true);
    $column(tableWorkEdit.$$.finishTimeM).setNullable(true);
    $column(tableWorkEdit.$$.breakTimeM).setNullable(true);
    $column(tableWorkEdit.$$.createDatetime).setNullable(true);
    $column(tableWorkEdit.$$.updateDatetime).setNullable(true);

  }

  /**
   * 対象の社員・日付でデータベースを検索します。
   *
   * @param empId 社員ID
   * @param date 日付
   * @throws Exception 例外発生時
   */
  public void loadDatabase(Long empId, String date) throws Exception {
    //コネクション取得
    Connection conn = getConnection();
    try {
      this.empId = empId;
      this.date = date;
      //パラメータを設定
      tableWorkEdit.getParams().setValue("EMP_ID", this.empId);
      tableWorkEdit.getParams().setValue("WORK_DATE", Date.valueOf(this.date.replace("/", "-")));
      tableWorkEmp.getParams().setValue("EMP_ID", this.empId);
      //アクティブ化
      tableWorkEdit.activate(conn);
      tableWorkEmp.activate(conn);
      //setupColumns();
      //0件 = 新規登録の場合
      if (tableWorkEdit.getRowCount() == 0) {
        //テーブルオブジェクトに1行追加
        tableWorkEdit.appendRow();
        //デフォルト値を設定
        tableWorkEdit.entity.empId = this.empId;
        tableWorkEdit.entity.date = Date.valueOf(this.date.replace("/", "-"));
        tableWorkEdit.entity.workTypeCode = new Short("1");
      }
    } finally {
      //コネクションを閉じる
      conn.close();
    }
  }


  /**
   * 稼働時間の計算をします。
   *
   * @throws Exception 例外発生時
   */
  public void calcWorkTime() throws Exception {
    //リクエスト内容のチェック
    workTime = tableWorkEdit.entity.getWorkTime();
    //Ajaxレスポンスを返却(UIの更新はない)
    writeAjaxResponse("");
  }

  /**
   * 勤務情報の登録・更新をします。
   *
   * @throws Exception 例外発生時
   */
  public void saveEmp() throws Exception {
    //コネクション取得
    Connection conn = getConnection();
    try {
      if (tableWorkEdit.getErrors().isEmpty() && !hasParamsError()) {
        //WORK_ID設定
        for (Row<WorkEdit> row : tableWorkEdit.visibleRows()) {
          //新規登録の場合
          if ( row.getStatus().equals(Row.Status.CREATING) ||
               row.getStatus().equals(Row.Status.CREATED) ) {
            updateWorkSeq.execute(conn, true, true);
            //WORK_IDを採番する
            tableWorkSeq.activate(conn);
            //採番したWORK_IDを設定
            row.entity.id = tableWorkSeq.entity.value;
          }
        }
        //勤務情報テーブルを更新する
        tableWorkEdit.resolveChanges(conn);
        //トランザクションをコミット
        conn.commit();
      }
    } catch (Exception x) {
      //トランザクションをロールバック
      conn.rollback();
      //例外時処理
      tableWorkEdit.addErrorMessage(x.getMessage());
      x.printStackTrace();
      throw new LSApplicationException(x);
    } finally {
      //コネクションを閉じる
      conn.close();
      //Ajaxレスポンスを返却(UIの更新はない)
      writeAjaxResponse("");
    }
  }

  /**
   * 入力チェックを行います。
   *
   * @return boolean エラー有無
   * @throws Exception 例外発生時
   */
  private boolean hasParamsError() throws Exception {
    boolean hasError = false;
    //開始時刻
    if (tableWorkEdit.entity.beginTimeM > tableWorkEdit.entity.finishTimeM) {
      tableWorkEdit.addErrorMessage(
          tableWorkEdit.getColumn("finishTimeM").getCaption() + "は" +
          tableWorkEdit.getColumn("beginTimeM").getCaption() + "よりも後の時刻を設定して下さい。");
      hasError = true;
    }
    //休憩時間
    if (!hasError) {
      if (tableWorkEdit.entity.breakTimeM > (tableWorkEdit.entity.finishTimeM - tableWorkEdit.entity.beginTimeM)) {
        tableWorkEdit.addErrorMessage(tableWorkEdit.getColumn("breakTimeM") + "が多すぎます。ちゃんと働きましょう。");
        hasError = true;
      }
    }
    return hasError;
  }

  /**
   * Tableの行確定イベントで各項目の自動設定します。
   *
   * @param  e テーブルイベント
   * @throws Exception 例外発生時
   */
  @SuppressWarnings("unchecked")
  @Override
  public void rowValidating(TableEvent e) throws Exception {
    if (e.getSource()==tableWorkEdit) {
      Row<WorkEdit> row = e.getRow();
      row.startEditing();
      if (row.entity.createDatetime == null) {
        row.entity.createDatetime = new java.sql.Timestamp(System.currentTimeMillis());
      }
      row.entity.updateDatetime = new java.sql.Timestamp(System.currentTimeMillis());
    }
  }

  /**
   * プルダウンの時リストを返します。
   *
   * @return 時リスト
   */
  private List<Map<String, String>> getHourList() {
    List<Map<String, String>> hourList = new ArrayList<Map<String, String>>();
    String pad = "";
    for (int i = 0; i < 24; i++) {
      Map<String, String> y = new HashMap<String, String>();
      if (i < 10) pad = "0";
      y.put("text", pad + Integer.toString(i));
      y.put("value", pad + Integer.toString(i));
      hourList.add(y);
      pad = "";
    }
    return hourList;
  }

  /**
   * プルダウンの分リストを返します。
   *
   * @return 分リスト
   */
  private List<Map<String, String>> getMinuteList() {
    List<Map<String, String>> minuteList = new ArrayList<Map<String, String>>();
    String pad = "";
    for (int i = 0; i < 60; i = i+15) {
      Map<String, String> y = new HashMap<String, String>();
      if (i < 10) pad = "0";
      y.put("text", pad + Integer.toString(i));
      y.put("value", pad + Integer.toString(i));
      minuteList.add(y);
      pad = "";
    }
    return minuteList;
  }

}
