package lsjsample;

import java.io.Serializable;
import java.sql.Connection;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import jp.littlesoft.web.SQLActionPage;
import jp.littlesoft.web.WebSQLTable;

/**
 * Little J-Programing Sample
 * 勤務表の登録画面
 * 
 * @author LittleSoft Corporation
 * @version 1.3.0
 */
@SuppressWarnings("serial")
public class WorkPage extends SQLActionPage {

  /**
   * ログイン情報(現在時刻)
   */
  public String now = null;

  /**
   * 対象年月リスト(セレクトボックス)
   */
  public List<Map<String, String>> monthList = getMonthList(6);

  /**
   * 日付リスト
   */
  public List<Map<String, String>> dateList = null;

  /**
   * 現在選択されている社員の社員ID
   */
  public Long empId = new Long(-1);
  
  /**
   * 現在選択されている社員の社員名
   */
  public String empName = "";
  
  /**
   * 初期表示状態か否か
   */
  public boolean isInitDisplay = true;
  
  /**
   * WebSQLTableオブジェクト(社員情報テーブル)
   * SELECT T_EMP.EMP_ID, T_EMP.NAME, T_EMP.DEPT_CODE FROM T_EMP ORDER BY T_EMP.NAME
   */
  public WebSQLTable<DB.Employee> tableEmp = new WebSQLTable<DB.Employee>() {{
    select($$.id, $$.name, $$.deptCode).orderBy($$.name).prepare();
  }};
  
  /**
   * WebSQLTableオブジェクト(所属マスタテーブル)
   * SELECT M_DEPT.DEPT_CODE, M_DEPT.DEPT_NAME FROM M_DEPT ORDER BY M_DEPT.DEPT_NAME
   */
  public WebSQLTable<DB.Department> tableDept = new WebSQLTable<DB.Department>() {{
    select($$.code, $$.name).orderBy($$.name).prepare();   
  }};
  
  
  /**
   * WebSQLTableオブジェクト(勤務情報テーブル)拡張ビュー
   */
  static public class WorkView extends DB.Work {
    public WorkView() {
      finishTimeM = new Integer(0);
      beginTimeM  = new Integer(0);
      breakTimeM  = new Integer(0);
    }
    public String workTypeName;
    public int getWorkTimeM()     { return finishTimeM - beginTimeM - breakTimeM; }
    public String getWorkTime()   { return getTimeString(getWorkTimeM()); }
    public String getBeginTime()  { return getTimeString(beginTimeM); }
    public String getFinishTime() { return getTimeString(finishTimeM); }
    public String getBreakTime()  { return getTimeString(breakTimeM); }
    @Override
    public MetaData<?> createMetaData() {
      return new MetaData<WorkView>();
    }
  }
  
  /**
   * WebSQLTableオブジェクト(勤務情報テーブル)
   * SELECT 
   *   T_WORK.*
   *  ,M_WORK_TYPE.WORK_TYPE_NAME WORK_TYPE_NAME
   * FROM T_WORK INNER JOIN M_WORK_TYPE 
   *   ON (T_WORK.WORK_TYPE_CODE = M_WORK_TYPE.WORK_TYPE_CODE)
   * WHERE T_WORK.EMP_ID = :EMP_ID 
   *   AND (T_WORK.WORK_DATE BETWEEN :MONTH_S AND :MONTH_E)
   * ORDER BY T_WORK.WORK_DATE   
   */
  public WebSQLTable<WorkView> tableWork = new WebSQLTable<WorkView>(){{
    DB.Work     work = $useDummy(DB.Work.class);
    DB.WorkType type = $useDummy(DB.WorkType.class);
    select(
      $(work).into($$), 
      $(type.name).into($$.workTypeName)
    )
    .from(work).innerJoin(type)
      .on(work.workTypeCode).is(type.code)
    .end
    .where(work.empId).is(":EMP_ID")
      .and($if(work.date).between(":MONTH_S", ":MONTH_E"))
    .end
    .orderBy(work.date)
    .prepare();
  }};

  /**
   * 勤務情報テーブル(集計)ビュー
   */
  static public class WorkSummary implements Serializable {
    public Long empId;
    public Long breakTimeM = new Long(0);
    public Long workTimeM = new Long(0);
    public String getWorkTime()  { return getTimeString(workTimeM.intValue()); }
    public String getBreakTime() { return getTimeString(breakTimeM.intValue()); }
  }
  /**
   * WebSQLTableオブジェクト(勤務情報テーブル(集計))
   * SELECT
   *   T_WORK.EMP_ID EMP_ID
   *   ,SUM(T_WORK.BREAK_TIME_M) BREAK_TIME_M
   *   ,SUM((T_WORK.FINISH_TIME_M - T_WORK.BEGIN_TIME_M - T_WORK.BREAK_TIME_M)) 
   *   WORK_TIME_M
   * FROM T_WORK
   * WHERE T_WORK.EMP_ID = :EMP_ID
   *   AND T_WORK.WORK_DATE BETWEEN :MONTH_S AND :MONTH_E
   * GROUP BY T_WORK.EMP_ID 
   */
  public WebSQLTable<WorkSummary> tableWorkSum = new WebSQLTable<WorkSummary>(){{
    DB.Work work = $useDummy(DB.Work.class);
    select(
      $(work.empId).into($$.empId),
      $(work.breakTimeM).sum().into($$.breakTimeM),
      $( $(work.finishTimeM)
         .subtract(work.beginTimeM)
         .subtract(work.breakTimeM) ).sum().into($$.workTimeM)
    )
    .from(work)
    .where(work.empId).is(":EMP_ID")
      .and(work.date).between(":MONTH_S", ":MONTH_E")
    .end
    .groupBy(work.empId)
    .prepare();    
  }};

  /**
   * Velocityテンプレートファイル(ようこそ画面)
   */
  private static String workTemplate = "WorkPage.html";

  /**
   * Velocityテンプレートファイル(ようこそ画面(勤務情報))
   */
  private static String workTableTemplate = "WorkPageTable.html";

  /**
   * コンストラクタ。
   * 
   * @throws Exception 例外発生時
   */
  public WorkPage() {
  }
  
  /**
   * コンストラクタ。
   * 
   * @param request リクエスト
   * @param response レスポンス
   * @throws Exception 例外発生時
   */
  public WorkPage(HttpServletRequest request, HttpServletResponse response) throws Exception {
    super(request, response);
    setUseAjax(true);              //Ajax呼び出しを使用します。
    setTemplateName(workTemplate); //HTMLテンプレートを指定します。
    setStrictSession(true);        //バックボタンによる不正なリクエストをキャンセルします。
    //コネクション取得
    Connection conn = getConnection();
    try {
      //データベースを検索し、結果をWebSQLTableオブジェクトに格納する
      tableEmp.activate(conn);
      tableDept.activate(conn);
    } finally {
      //コネクションを閉じる
      conn.close(); 
    }
  }
  
  /**
   * 各アクションメソッド開始前に呼ばれるメソッドです。
   * 
   * @param methodName メソッド名
   */
  @Override
  protected void onStartAction(String methodName) {
    SimpleDateFormat sdf = new SimpleDateFormat("yyyy'年'MM'月'dd'日'(E) HH:mm:ss");
    now = sdf.format(new java.util.Date());
  }
  
  /**
   * 勤務情報テーブルをDBから読込、テーブルオブジェクトに格納します。
   * 
   * @param  empId 社員ID
   * @param  dname 社員名
   * @param  month 対象月
   * @throws Exception 例外発生時
   */
  public void searchWorkList(String empId, String empName, String month) throws Exception {
    //コネクション取得
    Connection conn = getConnection();
    try {
      //テーブルオブジェクトを一旦閉じる
      tableWork.passivate();
      tableWorkSum.passivate();
      //社員が未選択＆年月リストの変更の場合は何もしない
      if (empId.equals("-1") && this.empId.intValue()==-1) return;
      //対象年月の日付リストを取得
      this.dateList = getDateList(month);
      //社員情報を設定
      if (!empId.equals("-1")) this.empId = Long.parseLong(empId);
      if (!empName.equals("")) this.empName = empName;
      //パラメータの年月の最終日を算出する
      Integer y = Integer.parseInt(month.substring(0, 4));
      Integer m = Integer.parseInt(month.substring(4, 6));
      Calendar cal = Calendar.getInstance();
      cal.set(y, m.intValue() - 1, 1);
      java.sql.Date monthStart = new java.sql.Date(cal.getTimeInMillis());
      cal.set(y, m.intValue() - 1, cal.getActualMaximum(Calendar.DAY_OF_MONTH));
      java.sql.Date monthEnd = new java.sql.Date(cal.getTimeInMillis());
      //パラメータを設定し、テーブルオブジェクトに格納
      tableWork.getParams().setValue("EMP_ID", this.empId);
      tableWork.getParams().setValue("MONTH_S", monthStart);
      tableWork.getParams().setValue("MONTH_E", monthEnd);
      tableWork.activate(conn);
      tableWorkSum.getParams().setValue("EMP_ID", this.empId);
      tableWorkSum.getParams().setValue("MONTH_S", monthStart);
      tableWorkSum.getParams().setValue("MONTH_E", monthEnd);
      tableWorkSum.activate(conn);
      isInitDisplay = false;
    } finally {
      //コネクションを閉じる
      conn.close();
      //Ajaxレスポンスを返却(テンプレートと変数をマージした内容)
      writeAjaxResponse(getMergedString(workTableTemplate));
    }
  }

  /**
   * 勤務情報入力ダイアログを開きます。
   * 
   * @param  date 日付
   * @throws Exception 例外発生時
   */
  @Override
  public void showDialog(String date) throws Exception {
    this.getRequest().setAttribute(SQLActionPage.RA_PARENT_PAGE, this);
    //ダイアログのActionPageオブジェクト生成
    WorkDialog dialog = new WorkDialog();
    //ダイアログの初期化
    dialog.init(getRequest(), getResponse());
    //ダイアログの親ActionPageを設定
    dialog.setParentPage(this);
    //ダイアログのデータベース読み込み
    dialog.loadDatabase(this.empId, date);
    //ダイアログの表示
    dialog.show();
  }
  
  /**
   * テーブルの再検索(Dialog終了時に使用)を行います。
   * 
   * @throws Exception 例外発生時
   */
  public void refresh() throws Exception {
    //コネクション取得
    Connection conn = getConnection();
    try {
      //テーブルオブジェクトを一旦閉じる
      tableWork.passivate();
      tableWorkSum.passivate();
      //再検索
      tableWork.activate(conn);
      tableWorkSum.activate(conn);
      //Ajaxレスポンスを返却(テンプレートと変数をマージした内容)
      writeAjaxResponse(getMergedString(workTableTemplate));
    } finally {
      //コネクションを閉じる
      conn.close(); 
    }
  }
  
  /**
   * プルダウンの年月リストを返します。
   * 
   * @param prevMonth 過去月数
   * @return 年リスト
   */
  private List<Map<String, String>> getMonthList(int prevMonth) {
    List<Map<String, String>> monthList = new ArrayList<Map<String, String>>();
    Calendar cal = Calendar.getInstance();
    cal.set(Calendar.DATE, 1);
    for (int i = 0; i < prevMonth; i++) {
      Map<String, String> y = new HashMap<String, String>();
      y.put("text", (new SimpleDateFormat("yyyy年MM月")).format(cal.getTime()));
      y.put("value", (new SimpleDateFormat("yyyyMM")).format(cal.getTime()));
      monthList.add(y);
      cal.add(Calendar.MONTH, -1);
    }
    return monthList;
  }
  
  /**
   * 対象月の日付リストを返します。
   * 
   * @param targetMonth 対象月(yyyyMM)
   * @return 日付リスト
   */
  private List<Map<String, String>> getDateList(String targetMonth) {
    List<Map<String, String>> dateList = new ArrayList<Map<String, String>>();
    String year  = targetMonth.substring(0, 4);
    String month = targetMonth.substring(4, 6);
    Calendar cal = Calendar.getInstance();
    cal.set(Integer.parseInt(year), Integer.parseInt(month)-1, 1);
    while (true) {
      Map<String, String> y = new HashMap<String, String>();
      y.put("text", (new SimpleDateFormat("yyyy/MM/dd")).format(cal.getTime()));
      y.put("value", (new SimpleDateFormat("yyyyMMdd")).format(cal.getTime()));
      dateList.add(y);
      cal.add(Calendar.DATE, 1);
      if (cal.get(Calendar.DATE) == 1) break;
    }
    return dateList;
  }

  
  /**
   * 分から時間表示に変換します。
   * 
   * @param minute 分
   * @return 時間文字列「HH:mm」
   */
  static String getTimeString(Integer minute) {
    if (minute==null) return "";
    String minuteH = String.valueOf(minute/60);
    String minuteM = String.valueOf(minute%60);
    if (minuteH.length() == 1) minuteH = "0" + minuteH;
    if (minuteM.length() == 1) minuteM = "0" + minuteM;
    return minuteH + ":" + minuteM;
  }
}
