package lsjsample;

import java.io.Serializable;
import java.sql.Connection;
import java.sql.Date;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import jp.littlesoft.data.exception.DataException;
import jp.littlesoft.web.SQLActionPage;
import jp.littlesoft.web.WebSQLTable;

/**
 * Little J-Programing Sample
 * 勤務表の集計画面
 * 
 * @author LittleSoft Corporation
 * @version 1.3.0
 */
@SuppressWarnings("serial")
public class WorkSummaryPage extends SQLActionPage {

  /**
   * ログイン情報(現在時刻)
   */
  public String now = null;

  /**
   * 対象年月文字列
   */
  public String monthString = "";
  
  /**
   * 対象年月リスト
   */
  public List<Map<String, String>> monthList = getMonthList(6);
  
  /**
   * 作業時間サマリークラス
   * @author littlesoft
   *
   */
  static public class WorkSummary implements Serializable {
    public String name;
    public Long id;
    public Integer time1M = new Integer(0);
    public Integer time2M = new Integer(0);
    public Integer time3M = new Integer(0);
    public Integer time4M = new Integer(0);
    public Integer time5M = new Integer(0);
    public Integer time6M = new Integer(0);
    public String getTime1() { return WorkPage.getTimeString(time1M); };
    public String getTime2() { return WorkPage.getTimeString(time2M); };
    public String getTime3() { return WorkPage.getTimeString(time3M); };
    public String getTime4() { return WorkPage.getTimeString(time4M); };
    public String getTime5() { return WorkPage.getTimeString(time5M); };
    public String getTime6() { return WorkPage.getTimeString(time6M); };
  }
  
  private static Long ZERO = new Long(0);

  /**
   * WebSQLTableオブジェクト(勤務情報テーブル：全集計)
   * SELECT
   *   'すべて' AS NAME
   *  ,SUM(WORK_TIME1_M) AS WORK_TIME1_M, SUM(WORK_TIME2_M) AS WORK_TIME2_M, SUM(WORK_TIME3_M) AS WORK_TIME3_M
   *  ,SUM(WORK_TIME4_M) AS WORK_TIME4_M, SUM(WORK_TIME5_M) AS WORK_TIME5_M, SUM(WORK_TIME6_M) AS WORK_TIME6_M
   *  FROM
   *  (SELECT
   *     SUM(FINISH_TIME_M - BEGIN_TIME_M - BREAK_TIME_M) AS WORK_TIME1_M, 0 AS WORK_TIME2_M, 0 AS WORK_TIME3_M
   *    ,0 AS WORK_TIME4_M, 0 AS WORK_TIME5_M, 0 AS WORK_TIME6_M
   *   FROM T_WORK W
   *   WHERE :MONTH1_S <= W.WORK_DATE AND W.WORK_DATE <= :MONTH1_E
   *   UNION
   *     :(中略)   
   *   UNION
   *   SELECT
   *     0 AS WORK_TIME1_M, 0 AS WORK_TIME2_M, 0 AS WORK_TIME3_M
   *    ,0 AS WORK_TIME4_M, 0 AS WORK_TIME5_M, SUM(FINISH_TIME_M - BEGIN_TIME_M - BREAK_TIME_M) AS WORK_TIME6_M
   *   FROM T_WORK W
   *   WHERE :MONTH6_S <= W.WORK_DATE AND W.WORK_DATE <= :MONTH6_E
   *  ) AS TMP");
   */
  public WebSQLTable<WorkSummary> tableWorkSummary1 = new WebSQLTable<WorkSummary>(){{
    DB.Work     w = $useDummy(DB.Work.class);
    WorkSummary s = $useDummy(WorkSummary.class);
    select(
      $("すべて").into($$.name),
      $(s.time1M).sum().into($$.time1M), $(s.time2M).sum().into($$.time2M),
      $(s.time3M).sum().into($$.time3M), $(s.time4M).sum().into($$.time4M),
      $(s.time5M).sum().into($$.time5M), $(s.time6M).sum().into($$.time6M)
    ).from(
      select(
        $($(w.finishTimeM).subtract(w.beginTimeM).subtract(w.breakTimeM)).sum().into(s.time1M), 
        $(ZERO).into(s.time2M), $(ZERO).into(s.time3M),
        $(ZERO).into(s.time4M), $(ZERO).into(s.time5M), $(ZERO).into(s.time6M) 
      ).from(w).where(w.date).between(":MONTH1_S", ":MONTH1_E").end
      .unionSelect(
        $(ZERO).into(s.time1M), 
        $($(w.finishTimeM).subtract(w.beginTimeM).subtract(w.breakTimeM)).sum().into(s.time2M), 
        $(ZERO).into(s.time3M), 
        $(ZERO).into(s.time4M),  $(ZERO).into(s.time5M), $(ZERO).into(s.time6M)
      ).from(w).where(w.date).between(":MONTH2_S", ":MONTH2_E").end
      .unionSelect(
        $(ZERO).into(s.time1M),  $(ZERO).into(s.time2M), 
        $($(w.finishTimeM).subtract(w.beginTimeM).subtract(w.breakTimeM)).sum().into(s.time3M), 
        $(ZERO).into(s.time4M), $(ZERO).into(s.time5M), $(ZERO).into(s.time6M)
      ).from(w).where(w.date).between(":MONTH3_S", ":MONTH3_E").end
      .unionSelect(
        $(ZERO).into(s.time1M), $(ZERO).into(s.time2M), $(ZERO).into(s.time3M), 
        $($(w.finishTimeM).subtract(w.beginTimeM).subtract(w.breakTimeM)).sum().into(s.time4M), 
        $(ZERO).into(s.time5M), $(ZERO).into(s.time6M)
      ).from(w).where(w.date).between(":MONTH4_S", ":MONTH4_E").end
      .unionSelect(
        $(ZERO).into(s.time1M), $(ZERO).into(s.time2M), $(ZERO).into(s.time3M), 
        $(ZERO).into(s.time4M), 
        $($(w.finishTimeM).subtract(w.beginTimeM).subtract(w.breakTimeM)).sum().into(s.time5M), 
        $(ZERO).into(s.time6M)
      ).from(w).where(w.date).between(":MONTH5_S", ":MONTH5_E").end
      .unionSelect(
        $(ZERO).into(s.time1M), $(ZERO).into(s.time2M), $(ZERO).into(s.time3M), 
        $(ZERO).into(s.time4M), $(ZERO).into(s.time5M), 
        $($(w.finishTimeM).subtract(w.beginTimeM).subtract(w.breakTimeM)).sum().into(s.time6M) 
      ).from(w).where(w.date).between(":MONTH6_S", ":MONTH6_E").end
      .alias(s)
    )
    .prepare();
  }};
  
  /**
   * WebSQLTableオブジェクト(勤務情報テーブル：部署集計)
   *SELECT 
   *  DEPT_NAME AS NAME
   * ,TMP.DEPT_CODE
   * ,SUM(WORK_TIME1_M) AS WORK_TIME1_M, SUM(WORK_TIME2_M) AS WORK_TIME2_M, SUM(WORK_TIME3_M) AS WORK_TIME3_M
   * ,SUM(WORK_TIME4_M) AS WORK_TIME4_M, SUM(WORK_TIME5_M) AS WORK_TIME5_M, SUM(WORK_TIME6_M) AS WORK_TIME6_M
   * FROM
   * (SELECT
   *    DEPT_CODE
   *   ,SUM(FINISH_TIME_M - BEGIN_TIME_M - BREAK_TIME_M) AS WORK_TIME1_M, 0 AS WORK_TIME2_M, 0 AS WORK_TIME3_M
   *   ,0 AS WORK_TIME4_M, 0 AS WORK_TIME5_M, 0 AS WORK_TIME6_M
   *  FROM T_WORK W INNER JOIN T_EMP E ON E.EMP_ID = W.EMP_ID
   *  WHERE :MONTH1_S <= W.WORK_DATE AND W.WORK_DATE <= :MONTH1_E
   *  GROUP BY DEPT_CODE
   *  UNION
   *     :(中略)   
   *  UNION
   *  SELECT
   *    DEPT_CODE
   *   ,0 AS WORK_TIME1_M, 0 AS WORK_TIME2_M, 0 AS WORK_TIME3_M
   *   ,0 AS WORK_TIME4_M, 0 AS WORK_TIME5_M, SUM(FINISH_TIME_M - BEGIN_TIME_M - BREAK_TIME_M) AS WORK_TIME6_M
   *  FROM T_WORK W INNER JOIN T_EMP E ON E.EMP_ID = W.EMP_ID
   *  WHERE :MONTH6_S <= W.WORK_DATE AND W.WORK_DATE <= :MONTH6_E
   *  GROUP BY DEPT_CODE
   * ) AS TMP INNER JOIN M_DEPT D ON TMP.DEPT_CODE = D.DEPT_CODE
   * GROUP BY DEPT_NAME, TMP.DEPT_CODE
   * ORDER BY TMP.DEPT_CODE;
   */
  public WebSQLTable<WorkSummary> tableWorkSummary2 = new WebSQLTable<WorkSummary>(){{
    DB.Employee   e = $useDummy(DB.Employee.class);
    DB.Department d = $useDummy(DB.Department.class);
    DB.Work       w = $useDummy(DB.Work.class);
    WorkSummary   s = $useDummy(WorkSummary.class);
    select(
      $(d.name).into($$.name), $(s.id).into($$.id),
      $(s.time1M).sum().into($$.time1M), $(s.time2M).sum().into($$.time2M),
      $(s.time3M).sum().into($$.time3M), $(s.time4M).sum().into($$.time4M),
      $(s.time5M).sum().into($$.time5M), $(s.time6M).sum().into($$.time6M)
    ).from(
      select(
        $(e.deptCode).into(s.id),
        $($(w.finishTimeM).subtract(w.beginTimeM).subtract(w.breakTimeM)).sum().into(s.time1M), 
        $(ZERO).into(s.time2M), $(ZERO).into(s.time3M),
        $(ZERO).into(s.time4M), $(ZERO).into(s.time5M), $(ZERO).into(s.time6M) 
      ).from(w)
        .innerJoin(e).on(w.empId).is(e.id).end
        .where(w.date).between(":MONTH1_S", ":MONTH1_E").end
        .groupBy(e.deptCode)
      .unionSelect(
        $(e.deptCode).into(s.id),
        $(ZERO).into(s.time1M), 
        $($(w.finishTimeM).subtract(w.beginTimeM).subtract(w.breakTimeM)).sum().into(s.time2M), 
        $(ZERO).into(s.time3M), 
        $(ZERO).into(s.time4M),  $(ZERO).into(s.time5M), $(ZERO).into(s.time6M)
      ).from(w)
        .innerJoin(e).on(w.empId).is(e.id).end
        .where(w.date).between(":MONTH2_S", ":MONTH2_E").end
        .groupBy(e.deptCode)
      .unionSelect(
        $(e.deptCode).into(s.id),
        $(ZERO).into(s.time1M),  $(ZERO).into(s.time2M), 
        $($(w.finishTimeM).subtract(w.beginTimeM).subtract(w.breakTimeM)).sum().into(s.time3M), 
        $(ZERO).into(s.time4M), $(ZERO).into(s.time5M), $(ZERO).into(s.time6M)
      ).from(w)
        .innerJoin(e).on(w.empId).is(e.id).end
        .where(w.date).between(":MONTH3_S", ":MONTH3_E").end
        .groupBy(e.deptCode)
      .unionSelect(
        $(e.deptCode).into(s.id),
        $(ZERO).into(s.time1M), $(ZERO).into(s.time2M), $(ZERO).into(s.time3M), 
        $($(w.finishTimeM).subtract(w.beginTimeM).subtract(w.breakTimeM)).sum().into(s.time4M), 
        $(ZERO).into(s.time5M), $(ZERO).into(s.time6M)
      ).from(w)
        .innerJoin(e).on(w.empId).is(e.id).end
        .where(w.date).between(":MONTH4_S", ":MONTH4_E").end
        .groupBy(e.deptCode)
      .unionSelect(
        $(e.deptCode).into(s.id),
        $(ZERO).into(s.time1M), $(ZERO).into(s.time2M), $(ZERO).into(s.time3M), 
        $(ZERO).into(s.time4M), 
        $($(w.finishTimeM).subtract(w.beginTimeM).subtract(w.breakTimeM)).sum().into(s.time5M), 
        $(ZERO).into(s.time6M)
      ).from(w)
        .innerJoin(e).on(w.empId).is(e.id).end
        .where(w.date).between(":MONTH5_S", ":MONTH5_E").end
        .groupBy(e.deptCode)
      .unionSelect(
        $(e.deptCode).into(s.id),
        $(ZERO).into(s.time1M), $(ZERO).into(s.time2M), $(ZERO).into(s.time3M), 
        $(ZERO).into(s.time4M), $(ZERO).into(s.time5M), 
        $($(w.finishTimeM).subtract(w.beginTimeM).subtract(w.breakTimeM)).sum().into(s.time6M) 
      ).from(w)
        .innerJoin(e).on(w.empId).is(e.id).end
        .where(w.date).between(":MONTH6_S", ":MONTH6_E").end
        .groupBy(e.deptCode)
      .alias(s)
    )
    .innerJoin(d).on(s.id).is(d.code).end
    .groupBy(d.name, s.id)
    .orderBy(s.id)
    .prepare();
  }};
  
  /**
   * WebSQLTableオブジェクト(勤務情報テーブル：社員集計)
   *SELECT 
   *  NAME
   * ,EMP_ID
   * ,SUM(WORK_TIME1_M) AS WORK_TIME1_M, SUM(WORK_TIME2_M) AS WORK_TIME2_M, SUM(WORK_TIME3_M) AS WORK_TIME3_M
   * ,SUM(WORK_TIME4_M) AS WORK_TIME4_M, SUM(WORK_TIME5_M) AS WORK_TIME5_M, SUM(WORK_TIME6_M) AS WORK_TIME6_M
   * FROM
   * (SELECT
   *    W.EMP_ID
   *   ,E.NAME
   *   ,SUM(FINISH_TIME_M - BEGIN_TIME_M - BREAK_TIME_M) AS WORK_TIME1_M, 0 AS WORK_TIME2_M, 0 AS WORK_TIME3_M
   *   ,0 AS WORK_TIME4_M, 0 AS WORK_TIME5_M, 0 AS WORK_TIME6_M
   *  FROM T_WORK W INNER JOIN T_EMP E ON E.EMP_ID = W.EMP_ID
   *  WHERE :MONTH1_S <= W.WORK_DATE AND W.WORK_DATE <= :MONTH1_E AND E.DEPT_CODE = :DEPT_CODE
   *  GROUP BY W.EMP_ID, E.NAME
   *  UNION
   *  : (中略)
   *  UNION
   *  SELECT
   *    W.EMP_ID
   *   ,E.NAME
   *   ,0 AS WORK_TIME1_M, 0 AS WORK_TIME2_M, 0 AS WORK_TIME3_M
   *   ,0 AS WORK_TIME4_M, 0 AS WORK_TIME5_M, SUM(FINISH_TIME_M - BEGIN_TIME_M - BREAK_TIME_M) AS WORK_TIME6_M
   *  FROM T_WORK W INNER JOIN T_EMP E ON E.EMP_ID = W.EMP_ID
   *  WHERE :MONTH6_S <= W.WORK_DATE AND W.WORK_DATE <= :MONTH6_E AND E.DEPT_CODE = :DEPT_CODE
   *  GROUP BY W.EMP_ID, E.NAME
   * ) AS TMP
   * GROUP BY EMP_ID, NAME
   * ORDER BY EMP_ID
   */
  public WebSQLTable<WorkSummary> tableWorkSummary3 = new WebSQLTable<WorkSummary>(){{
    DB.Employee   e = $useDummy(DB.Employee.class);
    DB.Work       w = $useDummy(DB.Work.class);
    WorkSummary   s = $useDummy(WorkSummary.class);
    select(
      $(s.name).into($$.name), $(s.id).into($$.id),
      $(s.time1M).sum().into($$.time1M), $(s.time2M).sum().into($$.time2M),
      $(s.time3M).sum().into($$.time3M), $(s.time4M).sum().into($$.time4M),
      $(s.time5M).sum().into($$.time5M), $(s.time6M).sum().into($$.time6M)
    ).from(
      select(
        $(e.name).into(s.name),
        $(e.id).into(s.id),
        $($(w.finishTimeM).subtract(w.beginTimeM).subtract(w.breakTimeM)).sum().into(s.time1M), 
        $(ZERO).into(s.time2M), $(ZERO).into(s.time3M),
        $(ZERO).into(s.time4M), $(ZERO).into(s.time5M), $(ZERO).into(s.time6M) 
      ).from(w)
        .innerJoin(e).on(w.empId).is(e.id).end
        .where(w.date).between(":MONTH1_S", ":MONTH1_E").and(e.deptCode).is(":deptCode").end
        .groupBy(e.name, e.id)
      .unionSelect(
        $(e.name).into(s.name),
        $(e.id).into(s.id),
        $(ZERO).into(s.time1M), 
        $($(w.finishTimeM).subtract(w.beginTimeM).subtract(w.breakTimeM)).sum().into(s.time2M), 
        $(ZERO).into(s.time3M), 
        $(ZERO).into(s.time4M),  $(ZERO).into(s.time5M), $(ZERO).into(s.time6M)
      ).from(w)
        .innerJoin(e).on(w.empId).is(e.id).end
        .where(w.date).between(":MONTH2_S", ":MONTH2_E").and(e.deptCode).is(":deptCode").end
        .groupBy(e.name, e.id)
      .unionSelect(
        $(e.name).into(s.name),
        $(e.id).into(s.id),
        $(ZERO).into(s.time1M),  $(ZERO).into(s.time2M), 
        $($(w.finishTimeM).subtract(w.beginTimeM).subtract(w.breakTimeM)).sum().into(s.time3M), 
        $(ZERO).into(s.time4M), $(ZERO).into(s.time5M), $(ZERO).into(s.time6M)
      ).from(w)
        .innerJoin(e).on(w.empId).is(e.id).end
        .where(w.date).between(":MONTH3_S", ":MONTH3_E").and(e.deptCode).is(":deptCode").end
        .groupBy(e.name, e.id)
      .unionSelect(
        $(e.name).into(s.name),
        $(e.id).into(s.id),
        $(ZERO).into(s.time1M), $(ZERO).into(s.time2M), $(ZERO).into(s.time3M), 
        $($(w.finishTimeM).subtract(w.beginTimeM).subtract(w.breakTimeM)).sum().into(s.time4M), 
        $(ZERO).into(s.time5M), $(ZERO).into(s.time6M)
      ).from(w)
        .innerJoin(e).on(w.empId).is(e.id).end
        .where(w.date).between(":MONTH4_S", ":MONTH4_E").and(e.deptCode).is(":deptCode").end
        .groupBy(e.name, e.id)
      .unionSelect(
        $(e.name).into(s.name),
        $(e.id).into(s.id),
        $(ZERO).into(s.time1M), $(ZERO).into(s.time2M), $(ZERO).into(s.time3M), 
        $(ZERO).into(s.time4M), 
        $($(w.finishTimeM).subtract(w.beginTimeM).subtract(w.breakTimeM)).sum().into(s.time5M), 
        $(ZERO).into(s.time6M)
      ).from(w)
        .innerJoin(e).on(w.empId).is(e.id).end
        .where(w.date).between(":MONTH5_S", ":MONTH5_E").and(e.deptCode).is(":deptCode").end
        .groupBy(e.name, e.id)
      .unionSelect(
        $(e.name).into(s.name),
        $(e.id).into(s.id),
        $(ZERO).into(s.time1M), $(ZERO).into(s.time2M), $(ZERO).into(s.time3M), 
        $(ZERO).into(s.time4M), $(ZERO).into(s.time5M), 
        $($(w.finishTimeM).subtract(w.beginTimeM).subtract(w.breakTimeM)).sum().into(s.time6M) 
      ).from(w)
        .innerJoin(e).on(w.empId).is(e.id).end
        .where(w.date).between(":MONTH6_S", ":MONTH6_E").and(e.deptCode).is(":deptCode").end
        .groupBy(e.name, e.id)
      .alias(s)
    )
    .groupBy(s.name, s.id)
    .orderBy(s.id)
    .prepare();
  }};

  /**
   * Velocityテンプレートファイル(勤務表の集計画面)
   */
  private static String workSummaryTemplate = "WorkSummaryPage.html";

  /**
   * Velocityテンプレートファイル(勤務表の集計画面(テーブル行))
   */
  private static String workSummaryTableTrTemplate = "WorkSummaryPageTableTr.html";

  /**
   * コンストラクタ。
   * 
   * @throws Exception 例外発生時
   */
  public WorkSummaryPage() {
  }
  
  /**
   * コンストラクタ。
   * 
   * @param request リクエスト
   * @param response レスポンス
   * @throws Exception 例外発生時
   */
  public WorkSummaryPage(HttpServletRequest request, HttpServletResponse response) throws Exception {
    super(request, response);
    setUseAjax(true);                     //Ajax呼び出しを使用します。
    setTemplateName(workSummaryTemplate); //HTMLテンプレートを指定します。
    setStrictSession(true);               //バックボタンによる不正なリクエストをキャンセルします。
    //SQLパラメータ設定
    setupParams();
    //データベース読み込み
    loadDatabase();
  }
  
  /**
   * データベースを読み込みます。
   * 
   * @throws Exception 例外発生時
   */
  private void loadDatabase() throws Exception {
    //コネクション取得
    Connection conn = getConnection();
    try {
      //データベースを検索し、結果をWebSQLTableオブジェクトに格納する
      tableWorkSummary1.activate(conn);
      tableWorkSummary2.activate(conn);
    } finally {
      //コネクションを閉じる
      conn.close(); 
    }
  }
  
  /**
   * 各アクションメソッド開始前に呼ばれるメソッドです。
   * 
   * @param methodName メソッド名
   */
  @Override
  protected void onStartAction(String methodName) {
    SimpleDateFormat sdf = new SimpleDateFormat("yyyy'年'MM'月'dd'日'(E) HH:mm:ss");
    now = sdf.format(new java.util.Date());
  }
  
  /**
   * 勤務情報テーブルをDBから読込、テーブルオブジェクトに格納します。
   * 
   * @param  empId 社員ID
   * @param  dname 社員名
   * @param  month 対象月
   * @throws Exception 例外発生時
   */
  public void searchEmpSummary(String deptCode) throws Exception {
    
    if (deptCode == null || deptCode.equals("") || deptCode.equals("null")) {
      writeAjaxResponse("");
      return;
    }
    
    //コネクション取得
    Connection conn = getConnection();
    try {
      //テーブルオブジェクトを一旦閉じる
      tableWorkSummary3.passivate();
      tableWorkSummary3.getParams().setValue("deptCode", new Long(deptCode));
      tableWorkSummary3.activate(conn);
    } finally {
      //コネクションを閉じる
      conn.close();
      //Ajaxレスポンスを返却(テンプレートと変数をマージした内容)
      writeAjaxResponse(getMergedString(workSummaryTableTrTemplate));
    }
  }
  
 
  /**
   * プルダウンの年月リストを返します。
   * 
   * @param prevMonth 過去月数
   * @return 年リスト
   */
  private ArrayList<Map<String, String>> getMonthList(int prevMonth) {
    String start = "";
    String end   = "";
    ArrayList<Map<String, String>> monthList = new ArrayList<Map<String, String>>();
    Calendar cal = Calendar.getInstance();
    cal.set(Calendar.DATE, 1);
    for (int i = 0; i < prevMonth; i++) {
      Map<String, String> y = new HashMap<String, String>();
      end = new SimpleDateFormat("yyyy年MM月").format(cal.getTime());
      if (i==0) start = end;
      y.put("text", end);
      y.put("value", (new SimpleDateFormat("yyyyMM")).format(cal.getTime()));
      monthList.add(y);
      cal.add(Calendar.MONTH, -1);
    }
    this.monthString = end + " ～ " + start;
    return monthList;
  }

  /**
   * WebSQLTableに年月日パラメータを設定する。
   */
  private void setupParams(WebSQLTable<WorkSummary> table)  throws DataException {
    String temp = null;
    table.passivate();
    for (int i=0; i<6; i++) {
      temp = monthList.get(i).get("value");
      Calendar cal = Calendar.getInstance();
      cal.set(Integer.parseInt(temp.substring(0,4)), Integer.parseInt(temp.substring(4)) - 1, 1);
      table.getParams().setValue("MONTH" + (i+1) + "_S", new Date(cal.getTimeInMillis()));
      cal.set(Calendar.DAY_OF_MONTH, cal.getActualMaximum(Calendar.DAY_OF_MONTH));
      table.getParams().setValue("MONTH" + (i+1) + "_E", new Date(cal.getTimeInMillis()));
    }
  }
  private void setupParams() throws DataException {
    setupParams(tableWorkSummary1);
    setupParams(tableWorkSummary2);
    setupParams(tableWorkSummary3);
    return;
  }
  
}
